package com.example.service.serviceImpl;


import com.example.db1.dao.ArchiveDetailDao;
import com.example.db1.dao.ArchiveMasterDao;
import com.example.db1.dao.ArchiveOtherDao;
import com.example.db2.dao.MedicaRecordDao;
import com.example.dto.ArchiveDetailDto;
import com.example.dto.ArchiveOtherExtDto;
import com.example.dto.UpdateOtherDao;
import com.example.service.MedicaRecordService;
import com.example.vo.MedicaRecordVo;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.apache.http.HttpEntity;
import org.apache.http.client.methods.CloseableHttpResponse;
import org.apache.http.client.methods.HttpGet;
import org.apache.http.impl.client.CloseableHttpClient;
import org.apache.http.impl.client.HttpClients;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;


import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.math.BigInteger;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.List;

/**
 * @ClassName MedicaRecordServiceImpl
 * @Description 重耗材采集实体类
 * @Author linjj
 * @Date 2025/3/25 12:55
 * @Version 1.0
 */
@Service
@Slf4j
public class MedicaRecordServiceImpl implements MedicaRecordService {

    @Autowired
    private MedicaRecordDao medicaRecordDao;
    @Autowired
    private ArchiveMasterDao archiveMasterDao;
    @Value("${savePath}")
    private String savePath;
    @Autowired
    private ArchiveDetailDao archiveDetailDao;
    @Autowired
    private ArchiveOtherDao archiveOtherDao;

    @Override
    public void gatherMedicaRecord() {
        //根据时间段查询当前时间需要采集的患者信息
        List<MedicaRecordVo> patientInfos = medicaRecordDao.getMedicaRecord();
        if (CollectionUtils.isEmpty(patientInfos)) {
            log.info("当前时间段内无需要采集的数据");
            return;
        }
        log.info("当前时间需要采集:" + patientInfos.size() + "份重耗材");
        PollingGather(patientInfos);
    }
    /**
    * @Author: linjj
    * @Description: 轮询采集
    * @DateTime: 2025/3/25 14:45
    * @Params: [patientInfos]
    * @Return void
    */
    private void PollingGather(List<MedicaRecordVo> patientInfos) {
        //更新任务表
        UpdateOtherDao updateOtherDao=new UpdateOtherDao();
        //轮询采集
        for (MedicaRecordVo patientInfo : patientInfos) {
            try {
                //记帐号
                String medicaRecord = patientInfo.getMedicaRecord();
                //住院次数
                String visitId = patientInfo.getVisitId();
                //http下载地址
                String pdfPath = patientInfo.getPdfPath();
                //文件表
                ArchiveDetailDto archiveDetailDto = new ArchiveDetailDto();
                //任务表
                ArchiveOtherExtDto archiveOtherExtDto = new ArchiveOtherExtDto();
                //使用yyyyMMddHHmmssSSS格式作为文件名,文件id
                Date date = new Date();
                SimpleDateFormat format = new SimpleDateFormat("yyyyMMddHHmmssSSS");
                String newDate = format.format(date);
                //生成保存地址路径
                String newSavePath = GenerateSavePath(medicaRecord, visitId,newDate);
                log.info("当前采集患者记帐号为:"+medicaRecord+",住院次数为:"+visitId);
                //根据记帐号住院次数查询患者是否存在归档数据库中
                List<String> masterIds = archiveMasterDao.getMasterId(medicaRecord, visitId);
                if (CollectionUtils.isEmpty(masterIds)) {
                    log.info("当前采集患者基础信息不存在归档数据库中采集失败记帐号为:"+medicaRecord+",住院次数为:"+visitId);
                    continue;
                }
                //查询任务表中C2字段跟时间是否都一致，一致不需要重新采集
                List<String> archiveOtherExtAndTime = archiveOtherDao.getArchiveOtherExtAndTime(patientInfo.getId(), patientInfo.getExportTime());
                if (!CollectionUtils.isEmpty(archiveOtherExtAndTime)){
                    log.info("已经存在文件表跳过采集:"+medicaRecord+",住院次数为:"+visitId);
                    continue;
                }
                //查询任务表中c2字段是否已经存在，存在直接下载文件，不存在新增任务
                List<String> archiveOtherExt = archiveOtherDao.getArchiveOtherExt(patientInfo.getId());
                if (CollectionUtils.isEmpty(archiveOtherExt)) {
                    archiveOtherExtDto.setId(newDate);
                    archiveOtherExtDto.setSycTime(patientInfo.getExportTime());
                    archiveOtherExtDto.setSycObj(patientInfo.getPdfPath());
                    archiveOtherExtDto.setOtherID(BigInteger.valueOf(0));
                    archiveOtherExtDto.setSysFlag(BigInteger.valueOf(21));
                    archiveOtherExtDto.setSysUpdateTime(new Date());
                    archiveOtherExtDto.setJzh(medicaRecord);
                    archiveOtherExtDto.setStime(new Date());
                    archiveOtherExtDto.setETime(new Date());
                    archiveOtherExtDto.setStatusFlag(BigInteger.valueOf(0));
                    archiveOtherExtDto.setPResult("未开始");
                    archiveOtherExtDto.setMID(masterIds.get(0));
                    archiveOtherExtDto.setC2(patientInfo.getId());
                    archiveOtherDao.addArchiveOtherExt(archiveOtherExtDto);
                }
                updateDetail(patientInfo);
                //下载图像
                if (!DownloadFile(pdfPath, newSavePath)){
                    log.info("当前文件下载失败记帐号为:"+medicaRecord+",住院次数为:"+visitId);
                    failOther(updateOtherDao, patientInfo);
                    continue;
                }
                //下载成功保存文件表信息
                archiveDetailDto.setId(newDate);
                archiveDetailDto.setPdfPath(newSavePath);
                archiveDetailDto.setMasterId(masterIds.get(0));
                archiveDetailDto.setUpLoadDateTime(new Date());
                archiveDetailDto.setAssortId("61");
                archiveDetailDto.setSource("重耗材采集服务");
                archiveDetailDto.setFlag("0");
                archiveDetailDto.setSys("1");
                archiveDetailDto.setTitle("高价值耗材清单");
                archiveDetailDto.setC1(patientInfo.getId());
                //插入到文件表中
                if (!archiveDetailDao.addArchiveDetail(archiveDetailDto)){
                    log.info("插入文件表失败，采集失败记帐号为:"+medicaRecord+",住院次数为:"+visitId);
                }
                log.info("插入文件表成功，采集成功记帐号为:"+medicaRecord+",住院次数为:"+visitId);
                succeedOther(updateOtherDao, patientInfo, newDate);
            } catch (Exception e) {
                //下载失败更新任务表中任务状态
                failOther(updateOtherDao, patientInfo);
                log.error("系统异常,下载失败记帐号为:"+patientInfo.getMedicaRecord()+",住院次数为:"+patientInfo.getVisitId()+"异常信息为:"+e.getMessage(),e);
            }
        }
    }

    private void updateDetail(MedicaRecordVo patientInfo) {
        //判断文件表是否有记录，有记录先删除后更新
        List<String> pdfPathS = archiveDetailDao.getPdfPath(patientInfo.getId());
        if (!CollectionUtils.isEmpty(pdfPathS)) {
            //不为空删除对应记录
            if (archiveDetailDao.delPdfPath(patientInfo.getId())){
                //删除对应文件
                for (String path:pdfPathS){
                    File file1 = new File(path);
                    try {
                        file1.delete(); // 删除照片
                    } catch (Exception e) {
                        e.printStackTrace();
                    }
                }
            }
        }
    }

    /**
    * @Author: linjj
    * @Description: 成功任务更新任务表方法
    * @DateTime: 2025/3/26 16:07
    * @Params: [updateOtherDao, patientInfo, newDate]
    * @Return void
    */
    private void succeedOther(UpdateOtherDao updateOtherDao, MedicaRecordVo patientInfo, String newDate) {
        //下载成功更新任务表中任务状态
        updateOtherDao.setSysUpdateTime(new Date());
        updateOtherDao.setStime(new Date());
        updateOtherDao.setETime(new Date());
        updateOtherDao.setStatusFlag(BigInteger.valueOf(3));
        updateOtherDao.setPResult("Done");
        updateOtherDao.setDID(newDate);
        updateOtherDao.setC1(newDate +".pdf");
        updateOtherDao.setC2(patientInfo.getId());
        archiveOtherDao.updateArchiveOtherExt(updateOtherDao);
    }

    /**
    * @Author: linjj
    * @Description: 失败任务更新任务表方法
    * @DateTime: 2025/3/26 16:07
    * @Params: [updateOtherDao, patientInfo]
    * @Return void
    */
    private void failOther(UpdateOtherDao updateOtherDao, MedicaRecordVo patientInfo) {
        //下载失败更新任务表中任务状态
        updateOtherDao.setSysUpdateTime(new Date());
        updateOtherDao.setStime(new Date());
        updateOtherDao.setETime(new Date());
        updateOtherDao.setStatusFlag(BigInteger.valueOf(2));
        updateOtherDao.setPResult("文件下载失败");
        updateOtherDao.setC2(patientInfo.getId());
        archiveOtherDao.updateArchiveOtherExt(updateOtherDao);
    }

    /**
    * @Author: linjj
    * @Description: 生成新的保存路径
    * @DateTime: 2025/3/25 13:58
    * @Params: [inpNo, visitId]
    * @Return java.lang.String
    */
    private String GenerateSavePath(String inpNo,String visitId,String newDate){
        //保存目录
        String saveDirectory=savePath+ File.separatorChar+inpNo+ File.separatorChar+visitId;
        //保存地址
        String newSavePath=saveDirectory+File.separatorChar + newDate + ".pdf";
        // 检查目录是否存在，不存在则创建目录
        File directory = new File(saveDirectory);
        if (!directory.exists()) {
            boolean isCreated = directory.mkdirs(); // 创建所有必需的父目录
            if (!isCreated) {
                // 处理创建目录失败的情况
               log.info("目录创建失败,下载失败记帐号为:"+inpNo+",住院次数为:"+visitId);
            }
        }
        return newSavePath;
    }
    /**
    * @Author: linjj
    * @Description: 文件下载
    * @DateTime: 2025/3/25 14:50
    * @Params: [fileURL, savePath]
    * @Return java.lang.Boolean
    */
    private Boolean DownloadFile(String fileURL,String savePath) {
        try {
            CloseableHttpClient httpClient = HttpClients.createDefault();
            try (CloseableHttpResponse response = httpClient.execute(new HttpGet(fileURL))) {
                HttpEntity entity = response.getEntity();
                try (InputStream in = entity.getContent();
                     FileOutputStream fileOutputStream = new FileOutputStream(savePath)) {
                    byte[] buffer = new byte[1024];
                    int bytesRead;
                    while ((bytesRead = in.read(buffer)) != -1) {
                        fileOutputStream.write(buffer, 0, bytesRead);
                    }
                    return true;
                }
            }
        } catch (Exception e) {
            log.error("文件下载失败"+e.getMessage(),e);
            return false;
        }
    }
}
